@@ -48,9 +48,6 @@
 
 QT_BEGIN_NAMESPACE
 
-// workaround for VC++ 6.0 linker bug (?)
-typedef bool(*LessThan)(const QPair<QListWidgetItem*,int>&,const QPair<QListWidgetItem*,int>&);
-
 class QListWidgetMimeData : public QMimeData
 {
     Q_OBJECT
@@ -78,7 +75,7 @@ void QListModel::clear()
     for (int i = 0; i < items.count(); ++i) {
         if (items.at(i)) {
             items.at(i)->d->theid = -1;
-            items.at(i)->view = 0;
+            items.at(i)->view = nullptr;
             delete items.at(i);
         }
     }
@@ -99,7 +96,7 @@ void QListModel::remove(QListWidgetItem *item)
     Q_ASSERT(row != -1);
     beginRemoveRows(QModelIndex(), row, row);
     items.at(row)->d->theid = -1;
-    items.at(row)->view = 0;
+    items.at(row)->view = nullptr;
     items.removeAt(row);
     endRemoveRows();
 }
@@ -159,11 +156,11 @@ void QListModel::insert(int row, const QStringList &labels)
 QListWidgetItem *QListModel::take(int row)
 {
     if (row < 0 || row >= items.count())
-        return 0;
+        return nullptr;
 
     beginRemoveRows(QModelIndex(), row, row);
     items.at(row)->d->theid = -1;
-    items.at(row)->view = 0;
+    items.at(row)->view = nullptr;
     QListWidgetItem *item = items.takeAt(row);
     endRemoveRows();
     return item;
@@ -189,8 +186,9 @@ int QListModel::rowCount(const QModelIndex &parent) const
     return parent.isValid() ? 0 : items.count();
 }
 
-QModelIndex QListModel::index(QListWidgetItem *item) const
+QModelIndex QListModel::index(const QListWidgetItem *item_) const
 {
+    QListWidgetItem *item = const_cast<QListWidgetItem *>(item_);
     if (!item || !item->view || static_cast<const QListModel *>(item->view->model()) != this
         || items.isEmpty())
         return QModelIndex();
@@ -229,6 +227,22 @@ bool QListModel::setData(const QModelIndex &index, const QVariant &value, int ro
     return true;
 }
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+bool QListModel::clearItemData(const QModelIndex &index)
+{
+    if (!checkIndex(index, CheckIndexOption::IndexIsValid))
+        return false;
+    QListWidgetItem *item = items.at(index.row());
+    const auto beginIter = item->d->values.cbegin();
+    const auto endIter = item->d->values.cend();
+    if (std::all_of(beginIter, endIter, [](const QWidgetItemData& data) -> bool { return !data.value.isValid(); }))
+        return true; //it's already cleared
+    item->d->values.clear();
+    emit dataChanged(index, index, QVector<int>{});
+    return true;
+}
+#endif
+
 QMap<int, QVariant> QListModel::itemData(const QModelIndex &index) const
 {
     QMap<int, QVariant> roles;
@@ -249,7 +263,7 @@ bool QListModel::insertRows(int row, int count, const QModelIndex &parent)
 
     beginInsertRows(QModelIndex(), row, row + count - 1);
     QListWidget *view = qobject_cast<QListWidget*>(QObject::parent());
-    QListWidgetItem *itm = 0;
+    QListWidgetItem *itm = nullptr;
 
     for (int r = row; r < row + count; ++r) {
         itm = new QListWidgetItem;
@@ -268,10 +282,10 @@ bool QListModel::removeRows(int row, int count, const QModelIndex &parent)
         return false;
 
     beginRemoveRows(QModelIndex(), row, row + count - 1);
-    QListWidgetItem *itm = 0;
+    QListWidgetItem *itm = nullptr;
     for (int r = row; r < row + count; ++r) {
         itm = items.takeAt(row);
-        itm->view = 0;
+        itm->view = nullptr;
         itm->d->theid = -1;
         delete itm;
     }
@@ -279,6 +293,37 @@ bool QListModel::removeRows(int row, int count, const QModelIndex &parent)
     return true;
 }
 
+/*!
+    \since 5.13
+    \reimp
+*/
+bool QListModel::moveRows(const QModelIndex &sourceParent, int sourceRow, int count, const QModelIndex &destinationParent, int destinationChild)
+{
+    if (sourceRow < 0
+        || sourceRow + count - 1 >= rowCount(sourceParent)
+        || destinationChild < 0
+        || destinationChild > rowCount(destinationParent)
+        || sourceRow == destinationChild
+        || sourceRow == destinationChild - 1
+        || count <= 0
+        || sourceParent.isValid()
+        || destinationParent.isValid()) {
+        return false;
+    }
+    if (!beginMoveRows(QModelIndex(), sourceRow, sourceRow + count - 1, QModelIndex(), destinationChild))
+        return false;
+
+    int fromRow = sourceRow;
+    if (destinationChild < sourceRow)
+        fromRow += count - 1;
+    else
+        destinationChild--;
+    while (count--)
+        items.move(fromRow, destinationChild);
+    endMoveRows();
+    return true;
+}
+
 Qt::ItemFlags QListModel::flags(const QModelIndex &index) const
 {
     if (!index.isValid() || index.row() >= items.count() || index.model() != this)
@@ -291,7 +336,7 @@ void QListModel::sort(int column, Qt::SortOrder order)
     if (column != 0)
         return;
 
-    emit layoutAboutToBeChanged();
+    emit layoutAboutToBeChanged({}, QAbstractItemModel::VerticalSortHint);
 
     QVector < QPair<QListWidgetItem*,int> > sorting(items.count());
     for (int i = 0; i < items.count(); ++i) {
@@ -300,7 +345,7 @@ void QListModel::sort(int column, Qt::SortOrder order)
         sorting[i].second = i;
     }
 
-    LessThan compare = (order == Qt::AscendingOrder ? &itemLessThan : &itemGreaterThan);
+    const auto compare = (order == Qt::AscendingOrder ? &itemLessThan : &itemGreaterThan);
     std::sort(sorting.begin(), sorting.end(), compare);
     QModelIndexList fromIndexes;
     QModelIndexList toIndexes;
@@ -315,7 +360,7 @@ void QListModel::sort(int column, Qt::SortOrder order)
     }
     changePersistentIndexList(fromIndexes, toIndexes);
 
-    emit layoutChanged();
+    emit layoutChanged({}, QAbstractItemModel::VerticalSortHint);
 }
 
 /**
@@ -337,7 +382,7 @@ void QListModel::ensureSorted(int column, Qt::SortOrder order, int start, int en
         sorting[i].second = start + i;
     }
 
-    LessThan compare = (order == Qt::AscendingOrder ? &itemLessThan : &itemGreaterThan);
+    const auto compare = (order == Qt::AscendingOrder ? &itemLessThan : &itemGreaterThan);
     std::sort(sorting.begin(), sorting.end(), compare);
 
     QModelIndexList oldPersistentIndexes = persistentIndexList();
@@ -411,10 +456,10 @@ QList<QListWidgetItem*>::iterator QListModel::sortedInsertionIterator(
     return std::lower_bound(begin, end, item, QListModelGreaterThan());
 }
 
-void QListModel::itemChanged(QListWidgetItem *item)
+void QListModel::itemChanged(QListWidgetItem *item, const QVector<int> &roles)
 {
-    QModelIndex idx = index(item);
-    emit dataChanged(idx, idx);
+    const QModelIndex idx = index(item);
+    emit dataChanged(idx, idx, roles);
 }
 
 QStringList QListModel::mimeTypes() const
@@ -443,7 +488,7 @@ QMimeData *QListModel::mimeData(const QModelIndexList &indexes) const
     return mimeData;
 }
 
-#ifndef QT_NO_DRAGANDDROP
+#if QT_CONFIG(draganddrop)
 bool QListModel::dropMimeData(const QMimeData *data, Qt::DropAction action,
                               int row, int column, const QModelIndex &index)
 {
@@ -462,7 +507,7 @@ Qt::DropActions QListModel::supportedDropActions() const
     const QListWidget *view = qobject_cast<const QListWidget*>(QObject::parent());
     return view->supportedDropActions();
 }
-#endif // QT_NO_DRAGANDDROP
+#endif // QT_CONFIG(draganddrop)
 
 /*!
     \class QListWidgetItem
@@ -544,24 +589,6 @@ Qt::DropActions QListModel::supportedDropActions() const
     Returns the list widget containing the item.
 */
 
-/*!
-    \fn void QListWidgetItem::setSelected(bool select)
-    \since 4.2
-
-    Sets the selected state of the item to \a select.
-
-    \sa isSelected()
-*/
-
-/*!
-    \fn bool QListWidgetItem::isSelected() const
-    \since 4.2
-
-    Returns \c true if the item is selected; otherwise returns \c false.
-
-    \sa setSelected()
-*/
-
 /*!
     \fn void QListWidgetItem::setHidden(bool hide)
     \since 4.2
@@ -596,14 +623,14 @@ Qt::DropActions QListModel::supportedDropActions() const
 
     \sa type()
 */
-QListWidgetItem::QListWidgetItem(QListWidget *view, int type)
-    : rtti(type), view(view), d(new QListWidgetItemPrivate(this)),
+QListWidgetItem::QListWidgetItem(QListWidget *listview, int type)
+    : rtti(type), view(listview), d(new QListWidgetItemPrivate(this)),
       itemFlags(Qt::ItemIsSelectable
                 |Qt::ItemIsUserCheckable
                 |Qt::ItemIsEnabled
                 |Qt::ItemIsDragEnabled)
 {
-    if (QListModel *model = (view ? qobject_cast<QListModel*>(view->model()) : 0))
+    if (QListModel *model = listModel())
         model->insert(model->rowCount(), this);
 }
 
@@ -623,16 +650,20 @@ QListWidgetItem::QListWidgetItem(QListWidget *view, int type)
 
     \sa type()
 */
-QListWidgetItem::QListWidgetItem(const QString &text, QListWidget *view, int type)
-    : rtti(type), view(0), d(new QListWidgetItemPrivate(this)),
+QListWidgetItem::QListWidgetItem(const QString &text, QListWidget *listview, int type)
+    : rtti(type), view(listview), d(new QListWidgetItemPrivate(this)),
       itemFlags(Qt::ItemIsSelectable
                 |Qt::ItemIsUserCheckable
                 |Qt::ItemIsEnabled
                 |Qt::ItemIsDragEnabled)
 {
-    setData(Qt::DisplayRole, text);
-    this->view = view;
-    if (QListModel *model = (view ? qobject_cast<QListModel*>(view->model()) : 0))
+    QListModel *model = listModel();
+    {
+        QSignalBlocker b(view);
+        QSignalBlocker bm(model);
+        setData(Qt::DisplayRole, text);
+    }
+    if (model)
         model->insert(model->rowCount(), this);
 }
 
@@ -654,17 +685,21 @@ QListWidgetItem::QListWidgetItem(const QString &text, QListWidget *view, int typ
     \sa type()
 */
 QListWidgetItem::QListWidgetItem(const QIcon &icon,const QString &text,
-                                 QListWidget *view, int type)
-    : rtti(type), view(0), d(new QListWidgetItemPrivate(this)),
+                                 QListWidget *listview, int type)
+    : rtti(type), view(listview), d(new QListWidgetItemPrivate(this)),
       itemFlags(Qt::ItemIsSelectable
                 |Qt::ItemIsUserCheckable
                 |Qt::ItemIsEnabled
                 |Qt::ItemIsDragEnabled)
 {
-    setData(Qt::DisplayRole, text);
-    setData(Qt::DecorationRole, icon);
-    this->view = view;
-    if (QListModel *model = (view ? qobject_cast<QListModel*>(view->model()) : 0))
+    QListModel *model = listModel();
+    {
+        QSignalBlocker b(view);
+        QSignalBlocker bm(model);
+        setData(Qt::DisplayRole, text);
+        setData(Qt::DecorationRole, icon);
+    }
+    if (model)
         model->insert(model->rowCount(), this);
 }
 
@@ -673,7 +708,7 @@ QListWidgetItem::QListWidgetItem(const QIcon &icon,const QString &text,
 */
 QListWidgetItem::~QListWidgetItem()
 {
-    if (QListModel *model = (view ? qobject_cast<QListModel*>(view->model()) : 0))
+    if (QListModel *model = listModel())
         model->remove(this);
     delete d;
 }
@@ -690,6 +725,9 @@ QListWidgetItem *QListWidgetItem::clone() const
     Sets the data for a given \a role to the given \a value. Reimplement this
     function if you need extra roles or special behavior for certain roles.
 
+    \note The default implementation treats Qt::EditRole and Qt::DisplayRole as
+    referring to the same data.
+
     \sa Qt::ItemDataRole, data()
 */
 void QListWidgetItem::setData(int role, const QVariant &value)
@@ -707,8 +745,12 @@ void QListWidgetItem::setData(int role, const QVariant &value)
     }
     if (!found)
         d->values.append(QWidgetItemData(role, value));
-    if (QListModel *model = (view ? qobject_cast<QListModel*>(view->model()) : 0))
-        model->itemChanged(this);
+    if (QListModel *model = listModel()) {
+        const QVector<int> roles((role == Qt::DisplayRole) ?
+                                    QVector<int>({Qt::DisplayRole, Qt::EditRole}) :
+                                    QVector<int>({role}));
+        model->itemChanged(this, roles);
+    }
 }
 
 /*!
@@ -770,7 +812,7 @@ void QListWidgetItem::write(QDataStream &out) const
     \sa data(), flags()
 */
 QListWidgetItem::QListWidgetItem(const QListWidgetItem &other)
-    : rtti(Type), view(0),
+    : rtti(Type), view(nullptr),
       d(new QListWidgetItemPrivate(this)),
       itemFlags(other.itemFlags)
 {
@@ -792,6 +834,15 @@ QListWidgetItem &QListWidgetItem::operator=(const QListWidgetItem &other)
     return *this;
 }
 
+/*!
+   \internal
+   returns the QListModel if a view is set
+ */
+QListModel *QListWidgetItem::listModel() const
+{
+    return (view ? qobject_cast<QListModel*>(view->model()) : nullptr);
+}
+
 #ifndef QT_NO_DATASTREAM
 
 /*!
@@ -886,12 +937,14 @@ QDataStream &operator>>(QDataStream &in, QListWidgetItem &item)
     \sa Qt::AlignmentFlag
 */
 
+#if QT_DEPRECATED_SINCE(5, 13)
 /*!
     \fn QColor QListWidgetItem::backgroundColor() const
     \obsolete
 
     This function is deprecated. Use background() instead.
 */
+#endif
 
 /*!
     \fn QBrush QListWidgetItem::background() const
@@ -902,6 +955,7 @@ QDataStream &operator>>(QDataStream &in, QListWidgetItem &item)
     \sa setBackground(), foreground()
 */
 
+#if QT_DEPRECATED_SINCE(5, 13)
 /*!
     \fn QColor QListWidgetItem::textColor() const
     \obsolete
@@ -910,6 +964,7 @@ QDataStream &operator>>(QDataStream &in, QListWidgetItem &item)
 
     This function is deprecated. Use foreground() instead.
 */
+#endif
 
 /*!
     \fn QBrush QListWidgetItem::foreground() const
@@ -939,10 +994,55 @@ QDataStream &operator>>(QDataStream &in, QListWidgetItem &item)
     \fn void QListWidgetItem::setSizeHint(const QSize &size)
     \since 4.1
 
-    Sets the size hint for the list item to be \a size. If no size hint is set,
-    the item delegate will compute the size hint based on the item data.
+    Sets the size hint for the list item to be \a size.
+    If no size hint is set or \a size is invalid, the item
+    delegate will compute the size hint based on the item data.
 */
 
+/*!
+    \fn void QListWidgetItem::setSelected(bool select)
+    \since 4.2
+
+    Sets the selected state of the item to \a select.
+
+    \sa isSelected()
+*/
+void QListWidgetItem::setSelected(bool select)
+{
+    const QListModel *model = listModel();
+    if (!model || !view->selectionModel())
+        return;
+    const QAbstractItemView::SelectionMode selectionMode = view->selectionMode();
+    if (selectionMode == QAbstractItemView::NoSelection)
+        return;
+    const QModelIndex index = model->index(this);
+    if (selectionMode == QAbstractItemView::SingleSelection)
+        view->selectionModel()->select(index, select
+                                       ? QItemSelectionModel::ClearAndSelect
+                                       : QItemSelectionModel::Deselect);
+    else
+        view->selectionModel()->select(index, select
+                                       ? QItemSelectionModel::Select
+                                       : QItemSelectionModel::Deselect);
+}
+
+/*!
+    \fn bool QListWidgetItem::isSelected() const
+    \since 4.2
+
+    Returns \c true if the item is selected; otherwise returns \c false.
+
+    \sa setSelected()
+*/
+bool QListWidgetItem::isSelected() const
+{
+    const QListModel *model = listModel();
+    if (!model || !view->selectionModel())
+        return false;
+    const QModelIndex index = model->index(this);
+    return view->selectionModel()->isSelected(index);
+}
+
 /*!
     \fn void QListWidgetItem::setFlags(Qt::ItemFlags flags)
 
@@ -950,9 +1050,10 @@ QDataStream &operator>>(QDataStream &in, QListWidgetItem &item)
 
     \sa Qt::ItemFlags
 */
-void QListWidgetItem::setFlags(Qt::ItemFlags aflags) {
+void QListWidgetItem::setFlags(Qt::ItemFlags aflags)
+{
     itemFlags = aflags;
-    if (QListModel *model = (view ? qobject_cast<QListModel*>(view->model()) : 0))
+    if (QListModel *model = listModel())
         model->itemChanged(this);
 }
 
@@ -1026,22 +1127,28 @@ void QListWidgetItem::setFlags(Qt::ItemFlags aflags) {
     \since 4.2
 
     Sets the background brush of the list item to the given \a brush.
+    Setting a default-constructed brush will let the view use the
+    default color from the style.
 
     \sa background(), setForeground()
 */
 
+#if QT_DEPRECATED_SINCE(5, 13)
 /*!
     \fn void QListWidgetItem::setTextColor(const QColor &color)
     \obsolete
 
     This function is deprecated. Use setForeground() instead.
 */
+#endif
 
 /*!
     \fn void QListWidgetItem::setForeground(const QBrush &brush)
     \since 4.2
 
     Sets the foreground brush of the list item to the given \a brush.
+    Setting a default-constructed brush will let the view use the
+    default color from the style.
 
     \sa foreground(), setBackground()
 */
@@ -1120,7 +1227,7 @@ void QListWidgetPrivate::_q_emitCurrentItemChanged(const QModelIndex &current,
     //persistentCurrent is invalid if something changed the model in response
     //to the currentItemChanged signal emission and the item was removed
     if (!persistentCurrent.isValid()) {
-        currentItem = 0;
+        currentItem = nullptr;
     }
 
     emit q->currentTextChanged(currentItem ? currentItem->text() : QString());
@@ -1193,7 +1300,7 @@ void QListWidgetPrivate::_q_dataChanged(const QModelIndex &topLeft,
     new current item and the item that was previously current.
 
     \sa QListWidgetItem, QListView, QTreeView, {Model/View Programming},
-        {Config Dialog Example}
+        {Tab Dialog Example}
 */
 
 /*!
@@ -1369,7 +1476,7 @@ void QListWidget::setSelectionModel(QItemSelectionModel *selectionModel)
 
 /*!
     Returns the item that occupies the given \a row in the list if one has been
-    set; otherwise returns 0.
+    set; otherwise returns \nullptr.
 
     \sa row()
 */
@@ -1378,7 +1485,7 @@ QListWidgetItem *QListWidget::item(int row) const
 {
     Q_D(const QListWidget);
     if (row < 0 || row >= d->model->rowCount())
-        return 0;
+        return nullptr;
     return d->listModel()->at(row);
 }
 
@@ -1436,7 +1543,7 @@ void QListWidget::insertItems(int row, const QStringList &labels)
 
 /*!
     Removes and returns the item from the given \a row in the list widget;
-    otherwise returns 0.
+    otherwise returns \nullptr.
 
     Items removed from a list widget will not be managed by Qt, and will need
     to be deleted manually.
@@ -1448,7 +1555,7 @@ QListWidgetItem *QListWidget::takeItem(int row)
 {
     Q_D(QListWidget);
     if (row < 0 || row >= d->model->rowCount())
-        return 0;
+        return nullptr;
     return d->listModel()->take(row);
 }
 
@@ -1672,7 +1779,7 @@ QWidget *QListWidget::itemWidget(QListWidgetItem *item) const
 
     This function should only be used to display static content in the place of
     a list widget item. If you want to display custom dynamic content or
-    implement a custom editor widget, use QListView and subclass QItemDelegate
+    implement a custom editor widget, use QListView and subclass QStyledItemDelegate
     instead.
 
     \sa itemWidget(), removeItemWidget(), {Delegate Classes}
@@ -1684,6 +1791,7 @@ void QListWidget::setItemWidget(QListWidgetItem *item, QWidget *widget)
     QAbstractItemView::setIndexWidget(index, widget);
 }
 
+#if QT_DEPRECATED_SINCE(5, 13)
 /*!
     Returns \c true if \a item is selected; otherwise returns \c false.
 
@@ -1693,9 +1801,7 @@ void QListWidget::setItemWidget(QListWidgetItem *item, QWidget *widget)
 */
 bool QListWidget::isItemSelected(const QListWidgetItem *item) const
 {
-    Q_D(const QListWidget);
-    QModelIndex index = d->listModel()->index(const_cast<QListWidgetItem*>(item));
-    return selectionModel()->isSelected(index);
+    return ((item && item->listWidget() == this) ? item->isSelected() : false);
 }
 
 /*!
@@ -1708,20 +1814,10 @@ bool QListWidget::isItemSelected(const QListWidgetItem *item) const
 */
 void QListWidget::setItemSelected(const QListWidgetItem *item, bool select)
 {
-    Q_D(QListWidget);
-    QModelIndex index = d->listModel()->index(const_cast<QListWidgetItem*>(item));
-
-    if (d->selectionMode == SingleSelection) {
-        selectionModel()->select(index, select
-                                 ? QItemSelectionModel::ClearAndSelect
-                                 : QItemSelectionModel::Deselect);
-    } else if (d->selectionMode != NoSelection) {
-        selectionModel()->select(index, select
-                                 ? QItemSelectionModel::Select
-                                 : QItemSelectionModel::Deselect);
-    }
-
+    if (item && item->listWidget() == this)
+        const_cast<QListWidgetItem*>(item)->setSelected(select);
 }
+#endif
 
 /*!
     Returns a list of all selected items in the list widget.
@@ -1757,6 +1853,7 @@ QList<QListWidgetItem*> QListWidget::findItems(const QString &text, Qt::MatchFla
     return items;
 }
 
+#if QT_DEPRECATED_SINCE(5, 13)
 /*!
     Returns \c true if the \a item is explicitly hidden; otherwise returns \c false.
 
@@ -1780,6 +1877,7 @@ void QListWidget::setItemHidden(const QListWidgetItem *item, bool hide)
 {
     setRowHidden(row(item), hide);
 }
+#endif
 
 /*!
     Scrolls the view if necessary to ensure that the \a item is visible.
@@ -1822,8 +1920,8 @@ QStringList QListWidget::mimeTypes() const
     \a items. The format used to describe the items is obtained from the
     mimeTypes() function.
 
-    If the list of items is empty, 0 is returned instead of a serialized empty
-    list.
+    If the list of items is empty, \nullptr is returned instead of a
+    serialized empty list.
 */
 #if QT_VERSION >= QT_VERSION_CHECK(6,0,0)
 QMimeData *QListWidget::mimeData(const QList<QListWidgetItem *> &items) const
@@ -1838,7 +1936,7 @@ QMimeData *QListWidget::mimeData(const QList<QListWidgetItem*> items) const
     // if non empty, it's called from the model's own mimeData
     if (cachedIndexes.isEmpty()) {
         cachedIndexes.reserve(items.count());
-        foreach (QListWidgetItem *item, items)
+        for (QListWidgetItem *item : items)
             cachedIndexes << indexFromItem(item);
 
         QMimeData *result = d->listModel()->internalMimeData();
@@ -1850,7 +1948,7 @@ QMimeData *QListWidget::mimeData(const QList<QListWidgetItem*> items) const
     return d->listModel()->internalMimeData();
 }
 
-#ifndef QT_NO_DRAGANDDROP
+#if QT_CONFIG(draganddrop)
 /*!
     Handles \a data supplied by an external drag and drop operation that ended
     with the given \a action in the given \a index. Returns \c true if \a data and
@@ -1925,7 +2023,7 @@ Qt::DropActions QListWidget::supportedDropActions() const
     Q_D(const QListWidget);
     return d->listModel()->QAbstractListModel::supportedDropActions() | Qt::MoveAction;
 }
-#endif // QT_NO_DRAGANDDROP
+#endif // QT_CONFIG(draganddrop)
 
 /*!
     Returns a list of pointers to the items contained in the \a data object. If
@@ -1942,14 +2040,28 @@ QList<QListWidgetItem*> QListWidget::items(const QMimeData *data) const
 
 /*!
     Returns the QModelIndex associated with the given \a item.
+
+    \note In Qt versions prior to 5.10, this function took a non-\c{const} \a item.
 */
 
-QModelIndex QListWidget::indexFromItem(QListWidgetItem *item) const
+QModelIndex QListWidget::indexFromItem(const QListWidgetItem *item) const
 {
     Q_D(const QListWidget);
     return d->listModel()->index(item);
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+/*!
+    \internal
+    \obsolete
+    \overload
+*/
+QModelIndex QListWidget::indexFromItem(QListWidgetItem *item) const
+{
+    return indexFromItem(const_cast<const QListWidgetItem *>(item));
+}
+#endif
+
 /*!
     Returns a pointer to the QListWidgetItem associated with the given \a index.
 */
@@ -1959,7 +2071,7 @@ QListWidgetItem *QListWidget::itemFromIndex(const QModelIndex &index) const
     Q_D(const QListWidget);
     if (d->isIndexValid(index))
         return d->listModel()->at(index.row());
-    return 0;
+    return nullptr;
 }
 
 /*!
