@@ -45,6 +45,7 @@
 #include <qdrawutil.h>
 #include <qevent.h>
 #include <qfontmetrics.h>
+#include <qproxystyle.h>
 #include <qwindow.h>
 #include <qscreen.h>
 #include <qmainwindow.h>
@@ -53,6 +54,8 @@
 #include <qdebug.h>
 
 #include <private/qwidgetresizehandler_p.h>
+#include <private/qstylesheetstyle_p.h>
+#include <qpa/qplatformtheme.h>
 
 #include "qdockwidget_p.h"
 #include "qmainwindowlayout_p.h"
@@ -64,16 +67,19 @@ extern QString qt_setWindowTitle_helperHelper(const QString&, const QWidget*); /
 // qmainwindow.cpp
 extern QMainWindowLayout *qt_mainwindow_layout(const QMainWindow *window);
 
+static const QMainWindow *mainwindow_from_dock(const QDockWidget *dock)
+{
+    for (const QWidget *p = dock->parentWidget(); p; p = p->parentWidget()) {
+        if (const QMainWindow *window = qobject_cast<const QMainWindow*>(p))
+            return window;
+    }
+    return nullptr;
+}
+
 static inline QMainWindowLayout *qt_mainwindow_layout_from_dock(const QDockWidget *dock)
 {
-    const QWidget *p = dock->parentWidget();
-    while (p) {
-        const QMainWindow *window = qobject_cast<const QMainWindow*>(p);
-        if (window)
-            return qt_mainwindow_layout(window);
-        p = p->parentWidget();
-    }
-    return Q_NULLPTR;
+    auto mainWindow = mainwindow_from_dock(dock);
+    return mainWindow ? qt_mainwindow_layout(mainWindow) : nullptr;
 }
 
 static inline bool hasFeature(const QDockWidgetPrivate *priv, QDockWidget::DockWidgetFeature feature)
@@ -120,15 +126,22 @@ class QDockWidgetTitleButton : public QAbstractButton
 public:
     QDockWidgetTitleButton(QDockWidget *dockWidget);
 
-    QSize sizeHint() const Q_DECL_OVERRIDE;
-    QSize minimumSizeHint() const Q_DECL_OVERRIDE
+    QSize sizeHint() const override;
+    QSize minimumSizeHint() const override
     { return sizeHint(); }
 
-    void enterEvent(QEvent *event) Q_DECL_OVERRIDE;
-    void leaveEvent(QEvent *event) Q_DECL_OVERRIDE;
-    void paintEvent(QPaintEvent *event) Q_DECL_OVERRIDE;
-};
+    void enterEvent(QEvent *event) override;
+    void leaveEvent(QEvent *event) override;
+    void paintEvent(QPaintEvent *event) override;
 
+protected:
+    bool event(QEvent *event) override;
+
+private:
+    QSize dockButtonIconSize() const;
+
+    mutable int m_iconSize = -1;
+};
 
 QDockWidgetTitleButton::QDockWidgetTitleButton(QDockWidget *dockWidget)
     : QAbstractButton(dockWidget)
@@ -136,14 +149,57 @@ QDockWidgetTitleButton::QDockWidgetTitleButton(QDockWidget *dockWidget)
     setFocusPolicy(Qt::NoFocus);
 }
 
+bool QDockWidgetTitleButton::event(QEvent *event)
+{
+    switch (event->type()) {
+    case QEvent::StyleChange:
+    case QEvent::ScreenChangeInternal:
+        m_iconSize = -1;
+        break;
+    default:
+        break;
+    }
+    return QAbstractButton::event(event);
+}
+
+static inline bool isWindowsStyle(const QStyle *style)
+{
+    // Note: QStyleSheetStyle inherits QWindowsStyle
+    const QStyle *effectiveStyle = style;
+
+#if QT_CONFIG(style_stylesheet)
+    if (style->inherits("QStyleSheetStyle"))
+      effectiveStyle = static_cast<const QStyleSheetStyle *>(style)->baseStyle();
+#endif
+#if !defined(QT_NO_STYLE_PROXY)
+    if (style->inherits("QProxyStyle"))
+      effectiveStyle = static_cast<const QProxyStyle *>(style)->baseStyle();
+#endif
+
+    return effectiveStyle->inherits("QWindowsStyle");
+}
+
+QSize QDockWidgetTitleButton::dockButtonIconSize() const
+{
+    if (m_iconSize < 0) {
+        m_iconSize = style()->pixelMetric(QStyle::PM_SmallIconSize, nullptr, this);
+        // Dock Widget title buttons on Windows where historically limited to size 10
+        // (from small icon size 16) since only a 10x10 XPM was provided.
+        // Adding larger pixmaps to the icons thus caused the icons to grow; limit
+        // this to qpiScaled(10) here.
+        if (isWindowsStyle(style()))
+            m_iconSize = qMin((10 * logicalDpiX()) / 96, m_iconSize);
+    }
+    return QSize(m_iconSize, m_iconSize);
+}
+
 QSize QDockWidgetTitleButton::sizeHint() const
 {
     ensurePolished();
 
-    int size = 2*style()->pixelMetric(QStyle::PM_DockWidgetTitleBarButtonMargin, 0, this);
+    int size = 2*style()->pixelMetric(QStyle::PM_DockWidgetTitleBarButtonMargin, nullptr, this);
     if (!icon().isNull()) {
-        int iconSize = style()->pixelMetric(QStyle::PM_SmallIconSize, 0, this);
-        QSize sz = icon().actualSize(QSize(iconSize, iconSize));
+        const QSize sz = icon().actualSize(dockButtonIconSize());
         size += qMax(sz.width(), sz.height());
     }
 
@@ -170,7 +226,7 @@ void QDockWidgetTitleButton::paintEvent(QPaintEvent *)
     opt.init(this);
     opt.state |= QStyle::State_AutoRaise;
 
-    if (style()->styleHint(QStyle::SH_DockWidget_ButtonsHaveFrame, 0, this))
+    if (style()->styleHint(QStyle::SH_DockWidget_ButtonsHaveFrame, nullptr, this))
     {
         if (isEnabled() && underMouse() && !isChecked() && !isDown())
             opt.state |= QStyle::State_Raised;
@@ -182,12 +238,11 @@ void QDockWidgetTitleButton::paintEvent(QPaintEvent *)
     }
 
     opt.icon = icon();
-    opt.subControls = 0;
-    opt.activeSubControls = 0;
+    opt.subControls = { };
+    opt.activeSubControls = { };
     opt.features = QStyleOptionToolButton::None;
     opt.arrowType = Qt::NoArrow;
-    int size = style()->pixelMetric(QStyle::PM_SmallIconSize, 0, this);
-    opt.iconSize = QSize(size, size);
+    opt.iconSize = dockButtonIconSize();
     style()->drawComplexControl(QStyle::CC_ToolButton, &opt, &p, this);
 }
 
@@ -212,9 +267,11 @@ QDockWidgetLayout::~QDockWidgetLayout()
 bool QDockWidgetLayout::nativeWindowDeco() const
 {
     bool floating = parentWidget()->isWindow();
+#if QT_CONFIG(tabbar)
     if (auto groupWindow =
             qobject_cast<const QDockWidgetGroupWindow *>(parentWidget()->parentWidget()))
         floating = floating || groupWindow->tabLayoutInfo();
+#endif
     return nativeWindowDeco(floating);
 }
 
@@ -254,12 +311,12 @@ QLayoutItem *QDockWidgetLayout::itemAt(int index) const
     int cnt = 0;
     for (int i = 0; i < item_list.count(); ++i) {
         QLayoutItem *item = item_list.at(i);
-        if (item == 0)
+        if (item == nullptr)
             continue;
         if (index == cnt++)
             return item;
     }
-    return 0;
+    return nullptr;
 }
 
 QLayoutItem *QDockWidgetLayout::takeAt(int index)
@@ -267,7 +324,7 @@ QLayoutItem *QDockWidgetLayout::takeAt(int index)
     int j = 0;
     for (int i = 0; i < item_list.count(); ++i) {
         QLayoutItem *item = item_list.at(i);
-        if (item == 0)
+        if (item == nullptr)
             continue;
         if (index == j) {
             item_list[i] = 0;
@@ -276,7 +333,7 @@ QLayoutItem *QDockWidgetLayout::takeAt(int index)
         }
         ++j;
     }
-    return 0;
+    return nullptr;
 }
 
 int QDockWidgetLayout::count() const
@@ -305,7 +362,7 @@ QSize QDockWidgetLayout::sizeFromContent(const QSize &content, bool floating) co
     const bool nativeDeco = nativeWindowDeco(floating);
 
     int fw = floating && !nativeDeco
-            ? w->style()->pixelMetric(QStyle::PM_DockWidgetFrameWidth, 0, w)
+            ? w->style()->pixelMetric(QStyle::PM_DockWidgetFrameWidth, nullptr, w)
             : 0;
 
     const int th = titleHeight();
@@ -324,11 +381,10 @@ QSize QDockWidgetLayout::sizeFromContent(const QSize &content, bool floating) co
     if (content.height() < 0)
         result.setHeight(-1);
 
-    int left, top, right, bottom;
-    w->getContentsMargins(&left, &top, &right, &bottom);
+    const QMargins margins = w->contentsMargins();
     //we need to subtract the contents margin (it will be added by the caller)
-    QSize min = w->minimumSize() - QSize(left + right, top + bottom);
-    QSize max = w->maximumSize() - QSize(left + right, top + bottom);
+    QSize min = w->minimumSize().shrunkBy(margins);
+    QSize max = w->maximumSize().shrunkBy(margins);
 
     /* A floating dockwidget will automatically get its minimumSize set to the layout's
        minimum size + deco. We're *not* interested in this, we only take minimumSize()
@@ -338,7 +394,7 @@ QSize QDockWidgetLayout::sizeFromContent(const QSize &content, bool floating) co
 
     uint explicitMin = 0;
     uint explicitMax = 0;
-    if (w->d_func()->extra != 0) {
+    if (w->d_func()->extra != nullptr) {
         explicitMin = w->d_func()->extra->explicitMinSize;
         explicitMax = w->d_func()->extra->explicitMaxSize;
     }
@@ -392,7 +448,7 @@ QSize QDockWidgetLayout::minimumSize() const
 QWidget *QDockWidgetLayout::widgetForRole(Role r) const
 {
     QLayoutItem *item = item_list.at(r);
-    return item == 0 ? 0 : item->widget();
+    return item == nullptr ? nullptr : item->widget();
 }
 
 QLayoutItem *QDockWidgetLayout::itemForRole(Role r) const
@@ -403,12 +459,12 @@ QLayoutItem *QDockWidgetLayout::itemForRole(Role r) const
 void QDockWidgetLayout::setWidgetForRole(Role r, QWidget *w)
 {
     QWidget *old = widgetForRole(r);
-    if (old != 0) {
+    if (old != nullptr) {
         old->hide();
         removeWidget(old);
     }
 
-    if (w != 0) {
+    if (w != nullptr) {
         addChildWidget(w);
         item_list[r] = new QWidgetItemV2(w);
         w->show();
@@ -449,8 +505,8 @@ int QDockWidgetLayout::minimumTitleWidth() const
 
     int titleHeight = this->titleHeight();
 
-    int mw = q->style()->pixelMetric(QStyle::PM_DockWidgetTitleMargin, 0, q);
-    int fw = q->style()->pixelMetric(QStyle::PM_DockWidgetFrameWidth, 0, q);
+    int mw = q->style()->pixelMetric(QStyle::PM_DockWidgetTitleMargin, nullptr, q);
+    int fw = q->style()->pixelMetric(QStyle::PM_DockWidgetFrameWidth, nullptr, q);
 
     return pick(verticalTitleBar, closeSize)
             + pick(verticalTitleBar, floatSize)
@@ -475,7 +531,7 @@ int QDockWidgetLayout::titleHeight() const
                             perp(verticalTitleBar, floatSize));
 
     QFontMetrics titleFontMetrics = q->fontMetrics();
-    int mw = q->style()->pixelMetric(QStyle::PM_DockWidgetTitleMargin, 0, q);
+    int mw = q->style()->pixelMetric(QStyle::PM_DockWidgetTitleMargin, nullptr, q);
 
     return qMax(buttonHeight + 2, titleFontMetrics.height() + 2*mw);
 }
@@ -487,7 +543,7 @@ void QDockWidgetLayout::setGeometry(const QRect &geometry)
     bool nativeDeco = nativeWindowDeco();
 
     int fw = q->isFloating() && !nativeDeco
-            ? q->style()->pixelMetric(QStyle::PM_DockWidgetFrameWidth, 0, q)
+            ? q->style()->pixelMetric(QStyle::PM_DockWidgetFrameWidth, nullptr, q)
             : 0;
 
     if (nativeDeco) {
@@ -611,6 +667,8 @@ void QDockWidgetPrivate::init()
     QObject::connect(button, SIGNAL(clicked()), q, SLOT(close()));
     layout->setWidgetForRole(QDockWidgetLayout::CloseButton, button);
 
+    font = QApplication::font("QDockWidgetTitle");
+
 #ifndef QT_NO_ACTION
     toggleViewAction = new QAction(q);
     toggleViewAction->setCheckable(true);
@@ -672,7 +730,7 @@ void QDockWidgetPrivate::updateButtons()
     QStyleOptionDockWidget opt;
     q->initStyleOption(&opt);
 
-    bool customTitleBar = dwLayout->widgetForRole(QDockWidgetLayout::TitleBar) != 0;
+    bool customTitleBar = dwLayout->widgetForRole(QDockWidgetLayout::TitleBar) != nullptr;
     bool nativeDeco = dwLayout->nativeWindowDeco();
     bool hideButtons = nativeDeco || customTitleBar;
 
@@ -719,18 +777,18 @@ void QDockWidgetPrivate::initDrag(const QPoint &pos, bool nca)
 {
     Q_Q(QDockWidget);
 
-    if (state != 0)
+    if (state != nullptr)
         return;
 
     QMainWindowLayout *layout = qt_mainwindow_layout_from_dock(q);
-    Q_ASSERT(layout != 0);
-    if (layout->pluggingWidget != 0) // the main window is animating a docking operation
+    Q_ASSERT(layout != nullptr);
+    if (layout->pluggingWidget != nullptr) // the main window is animating a docking operation
         return;
 
     state = new QDockWidgetPrivate::DragState;
     state->pressPos = pos;
     state->dragging = false;
-    state->widgetItem = 0;
+    state->widgetItem = nullptr;
     state->ownWidgetItem = false;
     state->nca = nca;
     state->ctrlDrag = false;
@@ -746,14 +804,14 @@ void QDockWidgetPrivate::startDrag(bool group)
 {
     Q_Q(QDockWidget);
 
-    if (state == 0 || state->dragging)
+    if (state == nullptr || state->dragging)
         return;
 
     QMainWindowLayout *layout = qt_mainwindow_layout_from_dock(q);
-    Q_ASSERT(layout != 0);
+    Q_ASSERT(layout != nullptr);
 
     state->widgetItem = layout->unplug(q, group);
-    if (state->widgetItem == 0) {
+    if (state->widgetItem == nullptr) {
         /* I have a QMainWindow parent, but I was never inserted with
             QMainWindow::addDockWidget, so the QMainWindowLayout has no
             widget item for me. :( I have to create it myself, and then
@@ -780,13 +838,14 @@ void QDockWidgetPrivate::startDrag(bool group)
 void QDockWidgetPrivate::endDrag(bool abort)
 {
     Q_Q(QDockWidget);
-    Q_ASSERT(state != 0);
+    Q_ASSERT(state != nullptr);
 
     q->releaseMouse();
 
     if (state->dragging) {
-        QMainWindowLayout *mwLayout = qt_mainwindow_layout_from_dock(q);
-        Q_ASSERT(mwLayout != 0);
+        const QMainWindow *mainWindow = mainwindow_from_dock(q);
+        Q_ASSERT(mainWindow != nullptr);
+        QMainWindowLayout *mwLayout = qt_mainwindow_layout(mainWindow);
 
         if (abort || !mwLayout->plug(state->widgetItem)) {
             if (hasFeature(this, QDockWidget::DockWidgetFloatable)) {
@@ -807,8 +866,12 @@ void QDockWidgetPrivate::endDrag(bool abort)
                 } else {
                     setResizerActive(false);
                 }
-                if (q->isFloating()) // Might not be floating when dragging a QDockWidgetGroupWindow
+                if (q->isFloating()) { // Might not be floating when dragging a QDockWidgetGroupWindow
                     undockedGeometry = q->geometry();
+#if QT_CONFIG(tabwidget)
+                    tabPosition = mwLayout->tabPosition(mainWindow->dockWidgetArea(q));
+#endif
+                }
                 q->activateWindow();
             } else {
                 // The tab was not plugged back in the QMainWindow but the QDockWidget cannot
@@ -818,7 +881,7 @@ void QDockWidgetPrivate::endDrag(bool abort)
         }
     }
     delete state;
-    state = 0;
+    state = nullptr;
 }
 
 void QDockWidgetPrivate::setResizerActive(bool active)
@@ -837,7 +900,7 @@ bool QDockWidgetPrivate::isAnimating() const
     Q_Q(const QDockWidget);
 
     QMainWindowLayout *mainWinLayout = qt_mainwindow_layout_from_dock(q);
-    if (mainWinLayout == 0)
+    if (mainWinLayout == nullptr)
         return false;
 
     return (const void*)mainWinLayout->pluggingWidget == (const void*)q;
@@ -862,14 +925,15 @@ bool QDockWidgetPrivate::mousePressEvent(QMouseEvent *event)
             // is not (but allow moving if the window is floating)
             (!hasFeature(this, QDockWidget::DockWidgetMovable) && !q->isFloating()) ||
             (qobject_cast<QMainWindow*>(parent) == 0 && !floatingTab) ||
-            isAnimating() || state != 0) {
+            isAnimating() || state != nullptr) {
             return false;
         }
 
         initDrag(event->pos(), false);
 
         if (state)
-            state->ctrlDrag = hasFeature(this, QDockWidget::DockWidgetFloatable) && event->modifiers() & Qt::ControlModifier;
+            state->ctrlDrag = (hasFeature(this, QDockWidget::DockWidgetFloatable) && event->modifiers() & Qt::ControlModifier) ||
+                              (!hasFeature(this, QDockWidget::DockWidgetMovable) && q->isFloating());
 
         return true;
     }
@@ -908,15 +972,11 @@ bool QDockWidgetPrivate::mouseMoveEvent(QMouseEvent *event)
     QMainWindowLayout *mwlayout = qt_mainwindow_layout_from_dock(q);
     if (!dwlayout->nativeWindowDeco()) {
         if (!state->dragging
-            && mwlayout->pluggingWidget == 0
+            && mwlayout->pluggingWidget == nullptr
             && (event->pos() - state->pressPos).manhattanLength()
                 > QApplication::startDragDistance()) {
             startDrag();
-#if 0 // Used to be included in Qt4 for Q_WS_WIN
-            grabMouseWhileInWindow();
-#else
             q->grabMouse();
-#endif
             ret = true;
         }
     }
@@ -959,18 +1019,11 @@ void QDockWidgetPrivate::nonClientAreaMouseEvent(QMouseEvent *event)
 {
     Q_Q(QDockWidget);
 
-    int fw = q->style()->pixelMetric(QStyle::PM_DockWidgetFrameWidth, 0, q);
+    int fw = q->style()->pixelMetric(QStyle::PM_DockWidgetFrameWidth, nullptr, q);
 
     QWidget *tl = q->topLevelWidget();
     QRect geo = tl->geometry();
     QRect titleRect = tl->frameGeometry();
-#if 0 // Used to be included in Qt4 for Q_WS_MAC
-    if ((features & QDockWidget::DockWidgetVerticalTitleBar)) {
-        titleRect.setTop(geo.top());
-        titleRect.setBottom(geo.bottom());
-        titleRect.setRight(geo.left() - 1);
-    } else
-#endif
     {
         titleRect.setLeft(geo.left());
         titleRect.setRight(geo.right());
@@ -982,20 +1035,21 @@ void QDockWidgetPrivate::nonClientAreaMouseEvent(QMouseEvent *event)
         case QEvent::NonClientAreaMouseButtonPress:
             if (!titleRect.contains(event->globalPos()))
                 break;
-            if (state != 0)
+            if (state != nullptr)
                 break;
             if (qobject_cast<QMainWindow*>(parent) == 0 && qobject_cast<QDockWidgetGroupWindow*>(parent) == 0)
                 break;
             if (isAnimating())
                 break;
             initDrag(event->pos(), true);
-            if (state == 0)
+            if (state == nullptr)
                 break;
-            state->ctrlDrag = event->modifiers() & Qt::ControlModifier;
+            state->ctrlDrag = (event->modifiers() & Qt::ControlModifier) ||
+                              (!hasFeature(this, QDockWidget::DockWidgetMovable) && q->isFloating());
             startDrag();
             break;
         case QEvent::NonClientAreaMouseMove:
-            if (state == 0 || !state->dragging)
+            if (state == nullptr || !state->dragging)
                 break;
 
 #ifndef Q_OS_MAC
@@ -1031,7 +1085,7 @@ void QDockWidgetPrivate::moveEvent(QMoveEvent *event)
 {
     Q_Q(QDockWidget);
 
-    if (state == 0 || !state->dragging || !state->nca)
+    if (state == nullptr || !state->dragging || !state->nca)
         return;
 
     if (!q->isWindow() && qobject_cast<QDockWidgetGroupWindow*>(parent) == 0)
@@ -1044,7 +1098,7 @@ void QDockWidgetPrivate::moveEvent(QMoveEvent *event)
         return;
 
     QMainWindowLayout *layout = qt_mainwindow_layout_from_dock(q);
-    Q_ASSERT(layout != 0);
+    Q_ASSERT(layout != nullptr);
 
     QPoint globalMousePos = event->pos() + state->pressPos;
     layout->hover(state->widgetItem, globalMousePos);
@@ -1118,6 +1172,8 @@ void QDockWidgetPrivate::setWindowState(bool floating, bool unplug, const QRect
             QMainWindowLayout *mwlayout = qt_mainwindow_layout_from_dock(q);
             if (mwlayout)
                 emit q->dockLocationChanged(mwlayout->dockWidgetArea(q));
+        } else {
+            emit q->dockLocationChanged(Qt::NoDockWidgetArea);
         }
     }
 
@@ -1328,7 +1384,7 @@ void QDockWidget::setFloating(bool floating)
     Q_D(QDockWidget);
 
     // the initial click of a double-click may have started a drag...
-    if (d->state != 0)
+    if (d->state != nullptr)
         d->endDrag(true);
 
     QRect r = d->undockedGeometry;
@@ -1419,10 +1475,11 @@ void QDockWidget::closeEvent(QCloseEvent *event)
 void QDockWidget::paintEvent(QPaintEvent *event)
 {
     Q_UNUSED(event)
+    Q_D(QDockWidget);
 
     QDockWidgetLayout *layout
         = qobject_cast<QDockWidgetLayout*>(this->layout());
-    bool customTitleBar = layout->widgetForRole(QDockWidgetLayout::TitleBar) != 0;
+    bool customTitleBar = layout->widgetForRole(QDockWidgetLayout::TitleBar) != nullptr;
     bool nativeDeco = layout->nativeWindowDeco();
 
     if (!nativeDeco && !customTitleBar) {
@@ -1439,6 +1496,11 @@ void QDockWidget::paintEvent(QPaintEvent *event)
         // the title may wish to extend out to all sides (eg. Vista style)
         QStyleOptionDockWidget titleOpt;
         initStyleOption(&titleOpt);
+        if (font() == QApplication::font("QDockWidget")) {
+            titleOpt.fontMetrics = QFontMetrics(d->font);
+            p.setFont(d->font);
+        }
+
         p.drawControl(QStyle::CE_DockWidgetTitle, titleOpt);
     }
 }
@@ -1454,7 +1516,7 @@ bool QDockWidget::event(QEvent *event)
     switch (event->type()) {
 #ifndef QT_NO_ACTION
     case QEvent::Hide:
-        if (layout != 0)
+        if (layout != nullptr)
             layout->keepSize(this);
         d->toggleViewAction->setChecked(false);
         emit visibilityChanged(false);
@@ -1463,10 +1525,10 @@ bool QDockWidget::event(QEvent *event)
         d->toggleViewAction->setChecked(true);
         QPoint parentTopLeft(0, 0);
         if (isWindow()) {
-            if (const QWindow *window = windowHandle())
-                parentTopLeft = window->screen()->availableVirtualGeometry().topLeft();
-            else
-                parentTopLeft = QGuiApplication::primaryScreen()->availableVirtualGeometry().topLeft();
+            const QScreen *screen = d->associatedScreen();
+            parentTopLeft = screen
+                ? screen->availableVirtualGeometry().topLeft()
+                : QGuiApplication::primaryScreen()->availableVirtualGeometry().topLeft();
         }
         emit visibilityChanged(geometry().right() >= parentTopLeft.x() && geometry().bottom() >= parentTopLeft.y());
 }
@@ -1480,12 +1542,14 @@ bool QDockWidget::event(QEvent *event)
         break;
     case QEvent::ZOrderChange: {
         bool onTop = false;
-        if (win != 0) {
+        if (win != nullptr) {
             const QObjectList &siblings = win->children();
             onTop = siblings.count() > 0 && siblings.last() == (QObject*)this;
         }
-        if (!isFloating() && layout != 0 && onTop)
+#if QT_CONFIG(tabbar)
+        if (!isFloating() && layout != nullptr && onTop)
             layout->raise(this);
+#endif
         break;
     }
     case QEvent::WindowActivate:
@@ -1512,17 +1576,6 @@ bool QDockWidget::event(QEvent *event)
         if (d->mouseMoveEvent(static_cast<QMouseEvent *>(event)))
             return true;
         break;
-#if 0 // Used to be included in Qt4 for Q_WS_WIN
-    case QEvent::Leave:
-        if (d->state != 0 && d->state->dragging && !d->state->nca) {
-            // This is a workaround for loosing the mouse on Vista.
-            QPoint pos = QCursor::pos();
-            QMouseEvent fake(QEvent::MouseMove, mapFromGlobal(pos), pos, Qt::NoButton,
-                             QApplication::mouseButtons(), QApplication::keyboardModifiers());
-            d->mouseMoveEvent(&fake);
-        }
-        break;
-#endif
     case QEvent::MouseButtonRelease:
         if (d->mouseReleaseEvent(static_cast<QMouseEvent *>(event)))
             return true;
@@ -1538,7 +1591,7 @@ bool QDockWidget::event(QEvent *event)
         break;
     case QEvent::Resize:
         // if the mainwindow is plugging us, we don't want to update undocked geometry
-        if (isFloating() && layout != 0 && layout->pluggingWidget != this)
+        if (isFloating() && layout != nullptr && layout->pluggingWidget != this)
             d->undockedGeometry = geometry();
 
         // Usually the window won't get resized while it's being moved, but it can happen,
@@ -1556,11 +1609,14 @@ bool QDockWidget::event(QEvent *event)
 
 #ifndef QT_NO_ACTION
 /*!
-  Returns a checkable action that can be used to show or close this
-  dock widget.
+  Returns a checkable action that can be added to menus and toolbars so that
+  the user can show or close this dock widget.
 
   The action's text is set to the dock widget's window title.
 
+  \note The action can not be used to programmatically show or hide the dock
+  widget. Use the \l visible property for that.
+
   \sa QAction::text, QWidget::windowTitle
  */
 QAction * QDockWidget::toggleViewAction() const
@@ -1618,8 +1674,8 @@ QAction * QDockWidget::toggleViewAction() const
     \since 4.3
 
     Sets an arbitrary \a widget as the dock widget's title bar. If \a widget
-    is 0, any custom title bar widget previously set on the dock widget is
-    removed, but not deleted, and the default title bar will be used
+    is \nullptr, any custom title bar widget previously set on the dock widget
+    is removed, but not deleted, and the default title bar will be used
     instead.
 
     If a title bar widget is set, QDockWidget will not use native window
@@ -1670,8 +1726,8 @@ void QDockWidget::setTitleBarWidget(QWidget *widget)
 
 /*!
     \since 4.3
-    Returns the custom title bar widget set on the QDockWidget, or 0 if no
-    custom title bar has been set.
+    Returns the custom title bar widget set on the QDockWidget, or
+    \nullptr if no custom title bar has been set.
 
     \sa setTitleBarWidget()
 */
